/* giFTcurs - curses interface to giFT
 * Copyright (C) 2001, 2002, 2003 Gran Weinholt <weinholt@dtek.chalmers.se>
 * Copyright (C) 2003 Christian Hggstrm <chm@c00.info>
 *
 * This file is part of giFTcurs.
 *
 * giFTcurs is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * giFTcurs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with giFTcurs; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,  USA.
 *
 * $Id: misc.c,v 1.23 2003/04/03 20:50:57 chnix Exp $
 */
#include "giftcurs.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <limits.h>

#include "misc.h"
#include "gift.h"
#include "protocol.h"
#include "poll.h"
#include "screen.h"
#include "ui.h"
#include "list.h"
#include "parse.h"

#define STATS_INTERVAL SECS(5)

gift_stat stats = { 0, 0, 0, 0, 0, 0 };
list messages = LIST_INITIALIZER;

static void stat_handle(ntree * packet, void (*callback) (void));
static void share_handle(ntree * packet, void *user_data);
static void attach_handle(ntree * packet, void *user_data);
static void detach_handle(ntree * packet, void (*callback) (void));
static void message_handle(ntree * packet, void *user_data);

static tick_t request_stats(void);
int sharing = -2;				/* 0 hide, 1 show, -1 progress, -2 initial */
static int will_change_network = 0;

void misc_init(void (*stat_callback) (void))
{
	gift_register("STATS", (EventCallback) stat_handle, stat_callback);
	gift_register("SHARE", (EventCallback) share_handle, NULL);
	gift_register("ATTACH", (EventCallback) attach_handle, NULL);
	gift_register("MESSAGE", (EventCallback) message_handle, NULL);
	gift_register("DETACH", (EventCallback) detach_handle, stat_callback);
}

void misc_destroy(void)
{
	/* This may look as a no-op, but it prevents further updates of the ui */
	gift_unregister("STATS", (EventCallback) stat_handle, NULL);
	gift_unregister("DETACH", (EventCallback) detach_handle, NULL);
	gift_register("STATS", (EventCallback) stat_handle, NULL);
	gift_register("DETACH", (EventCallback) detach_handle, NULL);
}

static void detach_handle(ntree * unused, void (*stat_callback) (void))
{
	free(stats.network);
	memset(&stats, 0, sizeof stats);
	poll_del_timer((TFunc) request_stats);
	if (stat_callback)
		stat_callback();
}

static void stat_add_protocol(ntree * tree, void *udata)
{
	unsigned int size, files, users;
	char *key = interface_name(tree);

	files = my_atoi(interface_lookup(tree, "files"));
	size = my_kilo_atof(interface_lookup(tree, "size"));
	users = my_atoi(interface_lookup(tree, "users"));

	if (!strcmp(key, "giFT")) {
		stats.own_files = files;
		stats.own_megs = size;
		return;
	}
	if (will_change_network) {
		if (!stats.network) {
			stats.network = strdup(key);
			stats.files = stats.megs = stats.users = 0;
			will_change_network = 0;
		} else if (!strcmp(key, stats.network)) {
			/* switch to the next network */
			free(stats.network);
			stats.network = NULL;
		}
	} else if (stats.network && strcmp(key, stats.network)) {
		return;
	}
	stats.files += files;
	stats.megs += size;
	stats.users += users;
}

static void stat_handle(ntree * packet, void (*callback) (void))
{
	/* STATS giFT { users(0) files(266) size(25.37) } OpenFT { users(0) files(0) size(0.00) }; */
	stats.files = stats.megs = stats.users = 0;

	interface_foreach(packet, (PForEachFunc) stat_add_protocol, NULL);

	will_change_network = 0;

	if (callback)
		callback();

	poll_add_timer(STATS_INTERVAL, (TFunc) request_stats, NULL);
}

static tick_t request_stats(void)
{
	/* Ask for stats, and starts the stats polling loop */
	ntree *packet = NULL;

	interface_append(&packet, "STATS", NULL);
	gift_write(&packet);
	return 0;
}

void stats_cycle(void)
{
	poll_del_timer((TFunc) request_stats);
	will_change_network = 1;
	request_stats();
}

static void share_handle(ntree * packet, void *user_data)
{
	char *action;

	if (!(action = interface_lookup(packet, "action"))) {
		DEBUG("no action");
		return;
	}

	if (!strcmp(action, "sync")) {
		char *status = interface_lookup(packet, "status");

		if (!status) {
			DEBUG("missing argument");
			return;
		}
		message(_("The daemon reported the following sync status: %s"), status);
	} else if (!strcmp(action, "show")) {
		if (sharing != -2 && sharing != 1) {
			sharing = 1;
			ui_draw();			/* just to update stats or button */
			message(_("Daemon reported that sharing now is enabled."));
		}
		sharing = 1;
	} else if (!strcmp(action, "hide")) {
		if (sharing != -2 && sharing != 0) {
			sharing = 0;
			ui_draw();
			message(_("Daemon reported that sharing now is disabled."));
		}
		sharing = 0;
	} else {
		DEBUG("missing argument");
	}
}

static void attach_handle(ntree * packet, void *user_data)
{
	static int first_time = 1;
	char *server, *version;

	server = interface_lookup(packet, "server");
	version = interface_lookup(packet, "version");

	if (!server || !version) {
		DEBUG("missing argument");
		return;
	}

	/* Skip the first, so that the notice about free software can be seen. */
	if (!first_time)
		message(_("Attached to %s v%s."), server, version);
	first_time = 0;

	/* Ask for the current share status. (hide or show) */
	packet = NULL;
	interface_append(&packet, "SHARE", NULL);
	interface_append(&packet, "action", NULL);
	gift_write(&packet);

	request_stats();
}

static void message_handle(ntree * packet, void *user_data)
{
	/* MESSAGE(x nodes have reported a more recent OpenFT revision than you are currently using.  You are STRONGLY advised to update your node as soon as possible.  See http://www.giftproject.org/ for more details.); */
	char *msg;

	msg = interface_value(packet);

	if (!msg)
		msg = _("never mind");

	asprintf(&msg, _("A message from the daemon:\n\n%s"), msg);
	list_append(&messages, msg);
	message(_("There is an important message from the daemon! It will be shown at exit."));
}

void sharing_toggle(void)
{
	ntree *packet = NULL;

	interface_append(&packet, "SHARE", NULL);
	interface_append(&packet, "action", sharing ? "hide" : "show");
	gift_write(&packet);

	sharing = -1;				/* unsure until we get acknowledge from daemon */
}

void sharing_sync(void)
{
	ntree *packet = NULL;

	interface_append(&packet, "SHARE", NULL);
	interface_append(&packet, "action", "sync");
	gift_write(&packet);
}
